'use strict';

var path = require('path');
var url = require('url');
var fs = require('fs');

/**
 * @description 是否是文件夹
 * @param path
 * @returns
 */
const isDir = (path) => {
    return fs.lstatSync(path).isDirectory();
};
/**
 * @description 是否存在文件
 * @param path
 * @returns
 */
const hasFile = (path) => {
    return fs.existsSync(path);
};
/**
 * @description 获取所有文件夹
 * @param path
 * @returns
 */
const getDirs = (path$1) => {
    const dirs = fs.readdirSync(path$1);
    return dirs.reduce((result, name) => {
        const fullPath = path.join(path$1, name);
        isDir(fullPath) && result.push({ dirName: name, dirPath: fullPath });
        return result;
    }, []);
};

/**
 * @description 获取json
 * @param jsonPath jsonPath
 * @returns
 */
function getJson(jsonPath) {
    try {
        const jsonText = fs.readFileSync(jsonPath, 'utf-8');
        return JSON.parse(jsonText);
    }
    catch (error) {
        process.exit(0);
    }
}
/**
 * @description 合并json
 * @param target IJson
 * @param source IJson
 * @returns IJson
 */
function mergeJson(target, source) {
    const targetPaths = target.compilerOptions?.paths ?? {};
    const sourcePaths = source.compilerOptions?.paths ?? {};
    for (const pathKey in targetPaths) {
        if (!Reflect.has(sourcePaths, pathKey)) {
            sourcePaths[pathKey] = targetPaths[pathKey];
        }
    }
    return {
        ...source,
        compilerOptions: {
            ...source.compilerOptions,
            paths: sourcePaths
        }
    };
}
/**
 * @description 生成json
 * @param alias { [key: string]: string }
 * @param root string
 * @param prefix string
 * @returns IJson
 */
function genJson(alias, root) {
    const { name: rootName } = path.parse(root);
    const paths = Object.keys(alias).reduce((result, pathKey) => {
        if (pathKey.startsWith('@') && pathKey !== '@') {
            const { name } = path.parse(alias[pathKey]);
            result[`${pathKey}/*`] = [`${rootName}/${name}/*`];
        }
        result['@/*'] = [`${rootName}/*`];
        return result;
    }, {});
    return {
        compilerOptions: {
            paths: paths,
            baseUrl: './'
        }
    };
}
/**
 * @description 同步json文件
 * @param extendJson 继承的json
 * @param jsJson jsconfig.json
 * @param tsJson tsconfig.json
 */
function syncJson({ extendJson, jsJson, tsJson, alias, mode, root }) {
    if (hasFile(extendJson) && ['all', 'extends'].includes(mode)) {
        const json = genJson(alias, root);
        hasFile(extendJson) && fs.writeFileSync(extendJson, JSON.stringify(json, null, 4));
    }
    if (hasFile(jsJson) && ['all', 'sync'].includes(mode)) {
        const target = genJson(alias, root);
        const source = getJson(jsJson);
        const newJson = mergeJson(target, source);
        hasFile(jsJson) && fs.writeFileSync(jsJson, JSON.stringify(newJson, null, 4));
    }
    if (hasFile(tsJson) && ['all', 'sync'].includes(mode)) {
        const target = genJson(alias, root);
        const source = getJson(tsJson);
        const newJson = mergeJson(target, source);
        hasFile(tsJson) && fs.writeFileSync(tsJson, JSON.stringify(newJson, null, 4));
    }
}

const __filename$1 = url.fileURLToPath((typeof document === 'undefined' ? require('u' + 'rl').pathToFileURL(__filename).href : (document.currentScript && document.currentScript.src || new URL('index.cjs.js', document.baseURI).href)));
const __dirname$1 = path.dirname(__filename$1);
const ALIAS_JSON_PATH = path.resolve(__dirname$1, '../alias.json');
const jsconfig = (root) => path.join(root, 'jsconfig.json');
const tsconfig = (root) => path.join(root, 'tsconfig.json');
function genAlias(root) {
    const dirs = getDirs(root);
    return dirs.reduce((result, item) => {
        const key = `@${item.dirName}`;
        const value = item.dirPath;
        result[key] = value;
        return result;
    }, {});
}
function alias(api) {
    const cwd = api.getCwd();
    const root = path.join(cwd, 'src');
    if (!hasFile(root)) {
        return;
    }
    api.configureWebpack(config => {
        let baseAlias = {};
        if (config.resolve && config.resolve.alias) {
            baseAlias = config.resolve.alias;
        }
        const _alias = { ...genAlias(root), ...baseAlias };
        syncJson({
            extendJson: ALIAS_JSON_PATH,
            jsJson: jsconfig(process.cwd()),
            tsJson: tsconfig(process.cwd()),
            alias: _alias,
            root,
            prefix: '@',
            mode: 'all'
        });
        return {
            resolve: {
                alias: _alias
            }
        };
    });
}

module.exports = alias;
